import { NgModule, Component, Compiler, Host, Input, Pipe, ElementRef, Directive, ViewContainerRef, ComponentFactoryResolver } from '@angular/core';
import { IonicModule, Content } from 'ionic-angular';
import * as _ from 'lodash';
export var MapToIterable = (function () {
    function MapToIterable() {
    }
    MapToIterable.prototype.transform = function (value) {
        var result = [];
        if (value.entries) {
            for (var _i = 0, _a = value.entries(); _i < _a.length; _i++) {
                var _b = _a[_i], key = _b[0], value = _b[1];
                result.push({ key: key, value: value });
            }
        }
        else {
            for (var key_1 in value) {
                result.push({ key: key_1, value: value[key_1] });
            }
        }
        return result;
    };
    MapToIterable.decorators = [
        { type: Pipe, args: [{ name: 'mapToIterable' },] },
    ];
    /** @nocollapse */
    MapToIterable.ctorParameters = [];
    return MapToIterable;
}());
export var DynamicHTMLOutlet = (function () {
    function DynamicHTMLOutlet(vcRef, resolver, compiler) {
        this.vcRef = vcRef;
        this.resolver = resolver;
        this.compiler = compiler;
    }
    DynamicHTMLOutlet.prototype.ngOnChanges = function (changes) {
        if (this.src) {
            this.addComponent(this.src, this.styles);
        }
    };
    DynamicHTMLOutlet.prototype.addComponent = function (template, styles) {
        var TemplateComponent = (function () {
            function TemplateComponent() {
            }
            TemplateComponent.decorators = [
                { type: Component, args: [{
                            selector: 'dynamic-html',
                            template: template,
                            styles: styles
                        },] },
            ];
            /** @nocollapse */
            TemplateComponent.ctorParameters = [];
            TemplateComponent.propDecorators = {
                'ionAlphaScrollRef': [{ type: Input },],
                'currentPageClass': [{ type: Input },],
            };
            return TemplateComponent;
        }());
        var TemplateModule = (function () {
            function TemplateModule() {
            }
            TemplateModule.decorators = [
                { type: NgModule, args: [{
                            imports: [IonicModule],
                            declarations: [TemplateComponent, MapToIterable]
                        },] },
            ];
            /** @nocollapse */
            TemplateModule.ctorParameters = [];
            return TemplateModule;
        }());
        var module = this.compiler.compileModuleAndAllComponentsSync(TemplateModule);
        var factory = _.find(module.componentFactories, function (comp) { return comp.componentType === TemplateComponent; });
        var componentRef = this.vcRef.createComponent(factory);
        componentRef.instance.ionAlphaScrollRef = this.ionAlphaScrollRef;
        componentRef.instance.currentPageClass = this.currentPageClass;
    };
    DynamicHTMLOutlet.decorators = [
        { type: Directive, args: [{
                    selector: 'dynamic-html-outlet',
                },] },
    ];
    /** @nocollapse */
    DynamicHTMLOutlet.ctorParameters = [
        { type: ViewContainerRef, },
        { type: ComponentFactoryResolver, },
        { type: Compiler, },
    ];
    DynamicHTMLOutlet.propDecorators = {
        'src': [{ type: Input },],
        'styles': [{ type: Input },],
        'ionAlphaScrollRef': [{ type: Input },],
        'currentPageClass': [{ type: Input },],
    };
    return DynamicHTMLOutlet;
}());
export var IonAlphaScroll = (function () {
    function IonAlphaScroll(_content, _elementRef, vcRef) {
        this._content = _content;
        this._elementRef = _elementRef;
        this.vcRef = vcRef;
        this.sortedItems = {};
        this.alphabet = [];
        this.ionAlphaScrollRef = this;
    }
    IonAlphaScroll.prototype.ngOnInit = function () {
        var _this = this;
        this.alphaScrollStyles = ["\n      .ion-alpha-sidebar {\n          position: fixed;\n          right: 0;\n          display: flex;\n          flex-flow: column;\n          z-index: 50000;\n        }\n\n        .ion-alpha-sidebar li {\n          flex: 1 1 auto;\n          list-style: none;\n          width: 15px;\n          text-align: center;\n        }\n    "];
        this.alphaScrollTemplate = "\n      <ion-scroll class=\"ion-alpha-scroll\" [ngStyle]=\"ionAlphaScrollRef.calculateScrollDimensions()\" scrollX=\"false\" scrollY=\"true\">\n        <ion-item-group class=\"ion-alpha-list-outer\">\n          <div *ngFor=\"let items of ionAlphaScrollRef.sortedItems | mapToIterable; trackBy:ionAlphaScrollRef.trackBySortedItems\">\n            <ion-item-divider id=\"scroll-letter-{{items.key}}\">{{items.key}}</ion-item-divider>\n            <div *ngFor=\"let item of items.value\">\n              " + this.itemTemplate + "\n            </div>\n          </div>\n        </ion-item-group>\n      </ion-scroll>\n      <ul class=\"ion-alpha-sidebar\" [ngStyle]=\"ionAlphaScrollRef.calculateDimensionsForSidebar()\">\n        <li *ngFor=\"let letter of ionAlphaScrollRef.alphabet\" tappable (click)=\"ionAlphaScrollRef.alphaScrollGoToList(letter)\">\n          <a>{{letter}}</a>\n        </li>\n      </ul>\n    ";
        setTimeout(function () {
            _this._scrollEle = _this._elementRef.nativeElement.querySelector('.scroll-content');
            _this.setupHammerHandlers();
        });
    };
    IonAlphaScroll.prototype.ngOnChanges = function (changes) {
        var tmp = {};
        for (var i = 0; i < this.listData.length; i++) {
            var listValue = _.get(this.listData[i], this.key);
            var letter = listValue.toUpperCase().charAt(0);
            if (typeof tmp[letter] === 'undefined') {
                tmp[letter] = [];
            }
            tmp[letter].push(this.listData[i]);
        }
        this.alphabet = this.iterateAlphabet(tmp);
        this.sortedItems = tmp;
    };
    IonAlphaScroll.prototype.calculateScrollDimensions = function () {
        var dimensions = this._content.getContentDimensions();
        return {
            height: dimensions.scrollHeight + 'px',
            width: (dimensions.contentWidth - 20) + 'px'
        };
    };
    IonAlphaScroll.prototype.calculateDimensionsForSidebar = function () {
        return {
            top: this._content.contentTop + 'px',
            height: (this._content.getContentDimensions().contentHeight - this._content.contentTop - 70) + 'px'
        };
    };
    IonAlphaScroll.prototype.alphaScrollGoToList = function (letter) {
        var ele = this._elementRef.nativeElement.querySelector("#scroll-letter-" + letter);
        var offsetY = ele.offsetTop;
        this._scrollEle.scrollTop = offsetY;
    };
    // create alphabet object
    IonAlphaScroll.prototype.iterateAlphabet = function (alphabet) {
        var str = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        var numbers = [];
        if (Object.keys(alphabet).length > 0) {
            str = '';
            for (var i = 0; i < Object.keys(alphabet).length; i++) {
                str += Object.keys(alphabet)[i];
            }
        }
        for (var i = 0; i < str.length; i++) {
            var nextChar = str.charAt(i);
            numbers.push(nextChar);
        }
        return numbers;
    };
    IonAlphaScroll.prototype.setupHammerHandlers = function () {
        var _this = this;
        var sidebarEle = this._elementRef.nativeElement.querySelector('.ion-alpha-sidebar');
        var mcHammer = new Hammer(sidebarEle, {
            recognizers: [
                // RecognizerClass, [options], [recognizeWith, ...], [requireFailure, ...]
                [Hammer.Pan, { direction: Hammer.DIRECTION_VERTICAL }],
            ]
        });
        mcHammer.on('panup pandown', _.throttle(function (e) {
            var closestEle = document.elementFromPoint(e.center.x, e.center.y);
            if (closestEle && ['LI', 'A'].indexOf(closestEle.tagName) > -1) {
                var letter = closestEle.innerText;
                var letterDivider = _this._elementRef.nativeElement.querySelector("#scroll-letter-" + letter);
                if (letterDivider) {
                    _this._scrollEle.scrollTop = letterDivider.offsetTop;
                }
            }
        }, 50));
    };
    IonAlphaScroll.prototype.trackBySortedItems = function (index, item) {
        return index;
    };
    IonAlphaScroll.decorators = [
        { type: Component, args: [{
                    selector: 'ion-alpha-scroll',
                    template: "\n    <dynamic-html-outlet\n      [src]=\"alphaScrollTemplate\"\n      [styles]=\"alphaScrollStyles\"\n      [ionAlphaScrollRef]=\"ionAlphaScrollRef\"\n      [currentPageClass]=\"currentPageClass\">\n    </dynamic-html-outlet>\n  "
                },] },
    ];
    /** @nocollapse */
    IonAlphaScroll.ctorParameters = [
        { type: Content, decorators: [{ type: Host },] },
        { type: ElementRef, },
        { type: ViewContainerRef, },
    ];
    IonAlphaScroll.propDecorators = {
        'listData': [{ type: Input },],
        'key': [{ type: Input },],
        'itemTemplate': [{ type: Input },],
        'currentPageClass': [{ type: Input },],
        'triggerChange': [{ type: Input },],
    };
    return IonAlphaScroll;
}());
//# sourceMappingURL=ion-alpha-scroll.js.map